<?php
/**
 * Created by PhpStorm.
 * User: roc
 * Date: 2019/12/19
 * Time: 17:01
 */
namespace kwy\wechatapi\openwx;

use kwy\util\http\CurlHttp;

class Component
{
    /**
     * @var CurlHttp
     */
    private $httpClient;

    private $component_appid = "";  //第三方平台appid

    private $component_appsecret = "";  //第三方平台appsecret

    private $component_verify_ticket = "";  //微信后台推送的ticket

    private $component_access_token = "";   //第三方平台令牌

    /**
     * 设置第三方平台appid
     * @author roc
     * @datetime:2019/12/19 17:02
     * @param string $component_appid
     * @return $this
     */
    public function setComponetAppid($component_appid){
        $this->component_appid = $component_appid;
        return $this;
    }

    /**
     * 获取第三方平台appid
     * @author roc
     * @datetime:2019/12/19 17:03
     * @return string
     */
    public function getComponetAppid(){
        return $this->component_appid;
    }

    /**
     * 设置第三方平台appsecret
     * @author roc
     * @datetime:2019/12/19 17:02
     * @param string $component_appsecret
     * @return $this
     */
    public function setComponetAppSecret($component_appsecret){
        $this->component_appsecret = $component_appsecret;
        return $this;
    }

    /**
     * 获取第三方平台appsecret
     * @author roc
     * @datetime:2019/12/19 17:03
     * @return string
     */
    public function getComponetAppSecret(){
        return $this->component_appsecret;
    }

    /**
     * 设置微信后台推送的ticket
     * @author roc
     * @datetime:2019/12/19 17:02
     * @param string $componentVerifyTicket
     * @return $this
     */
    public function setComponentVerifyTicket($componentVerifyTicket){
        $this->component_verify_ticket = $componentVerifyTicket;
        return $this;
    }

    /**
     * 获取微信后台推送的ticket
     * @author roc
     * @datetime:2019/12/19 17:03
     * @return string
     */
    public function getComponentVerifyTicket(){
        return $this->component_verify_ticket;
    }

    /**
     * 设置第三方平台令牌
     * @author roc
     * @datetime:2019/12/19 17:02
     * @param string $component_access_token
     * @return $this
     */
    public function setComponetAccessToken($component_access_token){
        $this->component_access_token = $component_access_token;
        return $this;
    }

    /**
     * 获取第三方平台令牌
     * @author roc
     * @datetime:2019/12/19 17:03
     * @return string
     */
    public function getComponetAccessToken(){
        return $this->component_access_token;
    }

    /**
     * 获取令牌
     * @author roc
     * @datetime:2019/12/19 17:07
     * @return array
     * @throws WxException
     */
    public function getApiComponentToken(){
        $data = [
            'component_appid' => $this->component_appid,
            'component_appsecret' => $this->component_appsecret,
            "component_verify_ticket" => $this->component_verify_ticket,
        ];
        $url = "https://api.weixin.qq.com/cgi-bin/component/api_component_token";

        /**
         * {"component_access_token":"28_VU8N5VkqE29Q0TfkTGSbc6o4ORv0o_Ta6vr0hRCB3zoWC0bEd33GXCUXHCjLGj0IEwiiuQE0fgWXMLMeLvybKM0DROqVzUAXTvz7IPPErrr5tMQR41vsKxsXAtvdBnoPfTn3-l4N1LtD7vYgUATeAAAGMF","expires_in":7200}'
         */
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 获取预授权码
     * @author roc
     * @datetime:2019/12/19 17:07
     * @return array
     * @throws WxException
     */
    public function getPreAuthCode(){
        $data = [
            'component_appid' => $this->component_appid,
        ];
        $url = "https://api.weixin.qq.com/cgi-bin/component/api_create_preauthcode?component_access_token=".$this->component_access_token;

        /**
         * 返回数据
         * {"pre_auth_code":"preauthcode@@@MxinwzxNEjW5Hkb60JCib4UuPpaCOvORrnOBuA7GQ-YtVeZBBXWxqCQLDYWMAsJA","expires_in":1800}
         */
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 使用授权码获取授权信息
     * @author roc
     * @datetime:2019/12/19 17:07
     * @param string $authorization_code 授权码, 会在授权成功时返回给第三方平台
     * @return mixed
     * @throws WxException
     */
    public function apiQueryAuth($authorization_code){
        $data = [
            'component_appid' => $this->component_appid,
            'authorization_code' => $authorization_code,
        ];
        $url = "https://api.weixin.qq.com/cgi-bin/component/api_query_auth?component_access_token=".$this->component_access_token;

        /**
         * 返回数据
         * {
        "authorization_info": {
        "authorizer_appid": "wx7a03580e7437b28f",
        "authorizer_access_token": "28_zf6oMMyhMvQDnjHiHra_qDzd8hdL5ySA1expLo2tiylhnWQTwInjdGSxwIadju2n1ZSodcNDQkKG89nRzfrt5D3wIFgYlEfnNnyDS_XwA-Z0spfo7JeTeE67fJ0ZAG0IMJTW1ggGGwXEVVvOOQRjAIDKMN",
        "expires_in": 7200,
        "authorizer_refresh_token": "refreshtoken@@@E9l-O9_UfStFJkyng58yvbsKOt2Mh4t3CMj576o0s7w",
        "func_info": [{
        "funcscope_category": {
        "id": 1
        }
        }, {
        "funcscope_category": {
        "id": 4
        }
        }, {
        "funcscope_category": {
        "id": 7
        }
        }, {
        "funcscope_category": {
        "id": 2
        }
        }, {
        "funcscope_category": {
        "id": 3
        }
        }, {
        "funcscope_category": {
        "id": 11
        }
        }, {
        "funcscope_category": {
        "id": 5
        }
        }, {
        "funcscope_category": {
        "id": 8
        }
        }, {
        "funcscope_category": {
        "id": 13
        }
        }, {
        "funcscope_category": {
        "id": 9
        }
        }]
        }
        }
         */
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 获取/刷新接口调用令牌
     * 在公众号/小程序接口调用令牌（authorizer_access_token）失效时，可以使用刷新令牌（authorizer_refresh_token）获取新的接口调用令牌。
     * 注意： authorizer_access_token 有效期为 2 小时，开发者需要缓存 authorizer_access_token，避免获取/刷新接口调用令牌的 API 调用触发每日限额。
     * @author roc
     * @datetime:2019/12/20 16:08
     * @param string $authorizer_appid 授权方appid
     * @param string $authorizer_refresh_token 刷新令牌，获取授权信息时得到
     * @return mixed
     * @throws WxException
     */
    public function apiAuthorizerToken($authorizer_appid,$authorizer_refresh_token){
        $data = [
            'component_appid' => $this->component_appid,
            'authorizer_appid' => $authorizer_appid,
            'authorizer_refresh_token' => $authorizer_refresh_token,
        ];
        $url = "https://api.weixin.qq.com/cgi-bin/component/api_authorizer_token?component_access_token=".$this->component_access_token;

        /**
         * 返回数据
         * {
        "authorizer_access_token": "28_9YNeF6QN0XkU6eAKtnsZO1upTKVT1yVE-9rdYPZl9JEdMSvlWMo0JY1Iosi-LvDF46oF_APlWeIpEQisH89smG1lFfl9bovJ4zk1sLdW3syExtrqitoNSuOayyq7ge5HV3FAg-UTSnTO23rCYJVhADDOZL",
        "expires_in": 7200,
        "authorizer_refresh_token": "refreshtoken@@@E9l-O9_UfStFJkyng58yvbsKOt2Mh4t3CMj576o0s7w"
        }
         */
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 获取授权方的帐号基本信息
     * @author roc
     * @datetime:2019/12/19 17:07
     * @param string $authorizer_appid 授权方appid
     * @return mixed
     * @throws WxException
     */
    public function apiGetAuthorizerInfo($authorizer_appid){
        $data = [
            'component_appid' => $this->component_appid,
            'authorizer_appid' => $authorizer_appid,
        ];
        $url = "https://api.weixin.qq.com/cgi-bin/component/api_get_authorizer_info?component_access_token=".$this->component_access_token;

        /**
         * 返回数据
         * {
        "authorizer_info": {
        "nick_name": "沪宁时代服务平台",
        "head_img": "http:\/\/wx.qlogo.cn\/mmopen\/W1hYCJNWG180eknPOLxuEfKxNPgr2joewJMn8RmqSmQ8hZyNPg6awvBbs1hJ7otDeYFZNk9G1IiaWM8XYxkT9ebHuaoxmDtt3\/0",
        "service_type_info": {
        "id": 2
        },
        "verify_type_info": {
        "id": 0
        },
        "user_name": "gh_56c70dec9ed1",
        "alias": "",
        "qrcode_url": "http:\/\/mmbiz.qpic.cn\/mmbiz_jpg\/a9lvM3o5Sm4bJDzMFbSQPkttr4m23PnBnfutK1FExiaxiaCiaoEiaBZ3aqtKo002yJIPEzgzA0lxKJqFUbcvxkIfGw\/0",
        "business_info": {
        "open_pay": 0,
        "open_shake": 0,
        "open_scan": 0,
        "open_card": 1,
        "open_store": 0
        },
        "idc": 1,
        "principal_name": "杭州萧山宁围沪宁副食品商店",
        "signature": "关注沪宁时代，即时收取特价商品介绍、门店活动、优惠券、电子会员、线上商城等。"
        },
        "authorization_info": {
        "authorizer_appid": "wx7a03580e7437b28f",
        "authorizer_refresh_token": "refreshtoken@@@E9l-O9_UfStFJkyng58yvbsKOt2Mh4t3CMj576o0s7w",
        "func_info": [{
        "funcscope_category": {
        "id": 1
        }
        }, {
        "funcscope_category": {
        "id": 4
        }
        }, {
        "funcscope_category": {
        "id": 7
        }
        }, {
        "funcscope_category": {
        "id": 2
        }
        }, {
        "funcscope_category": {
        "id": 3
        }
        }, {
        "funcscope_category": {
        "id": 11
        }
        }, {
        "funcscope_category": {
        "id": 5
        }
        }, {
        "funcscope_category": {
        "id": 8
        }
        }, {
        "funcscope_category": {
        "id": 13
        }
        }, {
        "funcscope_category": {
        "id": 9
        }
        }]
        }
        }
         */
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * post请求微信并返回array数据
     * @author roc
     * @datetime:2019/12/24 11:36
     * @param string $url 微信接口地址
     * @param array $data post请求的数据
     * @return mixed
     * @throws WxException
     */
    private function post($url,$data){
        $wx_response = $this->getHttpClient()
            ->setUrl($url)
            ->post(json_encode($data));
        if(empty($wx_response)) throw new \Exception("微信返回的数据为空");

        $result = json_decode($wx_response,true);
        if(empty($result)){
            throw new \Exception("转数组失败");
        }
        if(!empty($result["errcode"])){
            throw new WxException($result);
        }
        return $result;
    }

    /**
     * 设置代理
     * @author roc
     * @datetime:2019/12/8 14:37
     * @param $proxy
     * @return $this
     */
    public function setProxy($proxy){
        $this->getHttpClient()->setProxy($proxy);
        return $this;
    }

    /**
     * 获取http请求客户端
     * @author roc
     * @return CurlHttp
     */
    public function getHttpClient(){
        if (empty($this->httpClient)){
            $this->httpClient = new CurlHttp();
            //$this->httpClient->setProxy('127.0.0.1:8888');
            $this->httpClient->setContentType('application/json');
        }
        return $this->httpClient;
    }
}