<?php
/**
 * Created by PhpStorm.
 * User: roc
 * Date: 2019/12/19
 * Time: 17:01
 */
namespace kwy\wechatapi\openwx;

use kwy\util\http\CurlHttp;

class WxApp
{
    /**
     * @var CurlHttp
     */
    private $httpClient;

    /**
     * 第三方平台令牌
     * @var string
     */
    private $access_token = "";

    /**
     * 设置公众号平台令牌
     * @author roc
     * @datetime:2019/12/19 17:02
     * @param string $access_token
     * @return $this
     */
    public function setAccessToken($access_token){
        $this->access_token = $access_token;
        return $this;
    }

    /**
     * 获取第三方平台令牌
     * @author roc
     * @datetime:2019/12/19 17:03
     * @return string
     */
    public function getAccessToken(){
        return $this->access_token;
    }

    /**
     * 获取设置的行业信息
     * 获取帐号设置的行业信息。可登录微信公众平台，在公众号后台中查看行业信息。
     * 为方便第三方开发者，提供通过接口调用的方式来获取帐号所设置的行业信息，具体如下:
     * @author roc
     * @datetime:2019/12/20 18:17
     * @return mixed
     * @throws \Exception
     */
    public function getIndustry(){
        $data = [

        ];
        $url = "https://api.weixin.qq.com/cgi-bin/template/get_industry?access_token=".$this->access_token;

        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 设置所属行业
     * 设置行业可在微信公众平台后台完成，每月可修改行业1次，帐号仅可使用所属行业中相关的模板，
     * 为方便第三方开发者，提供通过接口调用的方式来修改账号所属行业，具体如下：
     * @author roc
     * @datetime:2019/12/20 18:17
     * @param int $industry_id1 公众号模板消息所属行业编号
     * @param int $industry_id2 公众号模板消息所属行业编号
     * @return mixed
     * @throws \Exception
     */
    public function apiSetIndustry($industry_id1,$industry_id2=0){
        $data = [
            "industry_id1" => $industry_id1
        ];
        if(!empty($industry_id2)) $data["industry_id2"] = $industry_id2;
        $url = "https://api.weixin.qq.com/cgi-bin/template/api_set_industry?access_token=".$this->access_token;

        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 获得模板ID
     * 从行业模板库选择模板到帐号后台，获得模板ID的过程可在微信公众平台后台完成。
     * 为方便第三方开发者，提供通过接口调用的方式来获取模板ID，具体如下：
     * @author roc
     * @datetime:2019/12/19 17:07
     * @param string $template_id_short 模板库中模板的编号，有“TM**”和“OPENTMTM**”等形式
     * @return mixed
     * @throws \Exception
     */
    public function apiAddTemplate($template_id_short){
        $data = [
            'template_id_short' => $template_id_short,
        ];
        $url = "https://api.weixin.qq.com/cgi-bin/template/api_add_template?access_token=".$this->access_token;

        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 发送模板消息
     * @author roc
     * @datetime:2019/12/20 16:34
     * @param string $openid
     * @param string $template_id
     * @param array $data
     *     @example ["first"=>["value"=>"恭喜你购买成功","color":"#173177"],"remark"=>["value"=>"备注1","color":"#173177"],keyword1]
     * @param string $url
     * @param array $miniprogram ["appid"=>"小程序appid","pagepath":"小程序页面路径"]
     *     @example ["appid"=>"xiaochengxuappid12345","pagepath":"index?foo=bar"]
     * @return mixed
     * @throws \Exception
     */
    public function sendTemplate($openid,$template_id,$data,$url="",$miniprogram=[]){
        $data = [
            'touser' => $openid,
            'template_id' => $template_id,
            'data' => $data
        ];
        if(!empty($url)) $data["url"] = $url;
        if(!empty($miniprogram)) $data["miniprogram"] = $miniprogram;

        $url = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=".$this->access_token;
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 获取模板列表
     * 获取已添加至帐号下所有模板列表，可在微信公众平台后台中查看模板列表信息。
     * 为方便第三方开发者，提供通过接口调用的方式来获取帐号下所有模板信息，具体如下:
     * @author roc
     * @datetime:2019/12/20 19:14
     * @return mixed
     * @throws \Exception
     */
    public function getAllPrivateTemplate(){
        $data = [];
        $url = "https://api.weixin.qq.com/cgi-bin/template/get_all_private_template?access_token=".$this->access_token;
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * post请求微信并返回array数据
     * @author roc
     * @datetime:2019/12/24 11:36
     * @param string $url 微信接口地址
     * @param array $data post请求的数据
     * @return mixed
     * @throws \Exception
     */
    private function post($url,$data){
        $wx_response = $this->getHttpClient()
            ->setUrl($url)
            ->post(json_encode($data));
        if(empty($wx_response)) throw new \Exception("微信返回的数据为空");

        $result = json_decode($wx_response,true);
        if(empty($result)){
            throw new \Exception("转数组失败");
        }
        if(!empty($result["errcode"])){
            throw new WxException($result);
        }
        return $result;
    }

    /**
     * 设置代理
     * @author roc
     * @datetime:2019/12/8 14:37
     * @param $proxy
     * @return $this
     */
    public function setProxy($proxy){
        $this->getHttpClient()->setProxy($proxy);
        return $this;
    }

    /**
     * 获取http请求客户端
     * @author roc
     * @return CurlHttp
     */
    public function getHttpClient(){
        if (empty($this->httpClient)){
            $this->httpClient = new CurlHttp();
            //$this->httpClient->setProxy('127.0.0.1:8888');
            $this->httpClient->setContentType('application/json');
        }
        return $this->httpClient;
    }
}