<?php
/**
 * Created by PhpStorm.
 * User: heyu
 * Date: 2019-06-04
 * Time: 16:54
 */

namespace kwy\wechatapi\wxapp;


use kwy\wechatapi\common\Request;

define('WXAPP_TYPE_MINI',0);
define('WXAPP_TYPE_SERVICE',1);

class Wxapp
{
    use Request;
    private $app_id;
    private $app_secret;
    private $type=0;

    /**
     * @return mixed
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * wxapp 类型 小程序 / 服务号
     * @param mixed $type WXAPP_TYPE_MINI  | WXAPP_TYPE_MINI
     */
    public function setType($type)
    {
        $this->type = $type;
    }

    /**
     * @param mixed $app_id
     */
    public function setAppId($app_id)
    {
        $this->app_id = $app_id;
    }

    /**
     * @param mixed $app_secret
     */
    public function setAppSecret($app_secret)
    {
        $this->app_secret = $app_secret;
    }

    /**
     * 向微信获取accesstoken
     * @param bool $origin 是否返回完整原文  默认false
     * @return string access_token 值
     * @throws \Exception
     */
    public function getAccessToken($origin=false){
        $url='https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=APPID&secret=APPSECRET';
        $url=str_replace(['APPID','APPSECRET'],[$this->app_id,$this->app_secret],$url);
        $ret=json_decode($this->get($url),true);
        if (!empty($ret['errmsg'])){
            throw new \Exception('获取accesstoken异常：'.$ret['errmsg']);
        }
        if ($origin){
            return $ret;
        }
        return $ret['access_token'];
    }

    /**
     * @param $code
     * @return mixed
     * @throws \Exception
     * @return array
     * @example
     * 小程序
     *   ["session_key"]=>"ZzdTuUadHO76M8ZzcCmp9w=="
     *   ["expires_in"]=>7200
     *   ["openid"]=>"o05oI0ZY-fY9BYAImYOd2yOHoHQQ"
     * 服务号
     *  "access_token"=>"ACCESS_TOKEN",
     *  "expires_in"=>7200,
     *  "refresh_token"=>"REFRESH_TOKEN",
     *  "openid"=>"OPENID",
     *  "scope"=>"SCOPE"
     */
    public function getOpenidByCode($code){
        if ($this->type==WXAPP_TYPE_MINI){
            $url='https://api.weixin.qq.com/sns/jscode2session?appid=APPID&secret=APPSECRET&js_code='.$code.'&grant_type=authorization_code';
        }elseif ($this->type==WXAPP_TYPE_SERVICE){
            $url='https://api.weixin.qq.com/sns/oauth2/access_token?appid=APPID&secret=APPSECRET&code='.$code.'&grant_type=authorization_code';
        }else{
            throw new \Exception('错误的WXAPP_TYPE');
        }
        $url=str_replace(['APPID','APPSECRET'],[$this->app_id,$this->app_secret],$url);
        $ret=json_decode($this->get($url),true);
        if (!empty($ret['errcode'])){
            throw new \Exception($ret['errcode'].','.$ret['errmsg']);
        }
        return $ret;
    }



}